/**
 * An angular filter. They can be registered through the [[ProjectSettings]] setup.
 * You must give a name, and the type for the filter.
 * Example:
 * ```new NgFilter("translate", "(key: string) => string")```
 *
 * You can also supply a addScopeToParam function: you are given the parameter index,
 * and are called with the parameter, so you can or not add scope accessors.
 * 99% of the time you want scope accessors for filter parameters, but for instance
 * 'filter' allows you to specify a pattern for items through an object literal.
 * If the filter spec is {name: "test"}, we certainly don't want to change it to:
 * {$scope.name: "test"}.
 */
export declare class NgFilter {
    readonly name: string;
    readonly type: string;
    readonly addScopeToParam: (paramIndex: number, input: string, addScopeAccessors: (input: string) => string) => string;
    /**
     * @param name The name of the angular filter
     * @param type The type that'll be used to type-check uses of the filter.
     * @param addScopeToParam Customize the scope adding for parameter index n For instance
     *        for 'filter', we shouldn't wrap the values in the scope for the
     *        'expression' parameter, which is a spec of the values to keep.
     */
    constructor(name: string, type: string, addScopeToParam?: (paramIndex: number, input: string, addScopeAccessors: (input: string) => string) => string);
}
/**
 * Set of angular filters supported out of the box. You can give this list in
 * [[ProjectSettings.ngFilters]], or you can add your own or provide your own
 * list entirely.
 */
export declare const defaultNgFilters: NgFilter[];
