"use strict";
exports.__esModule = true;
var prelude_ts_1 = require("prelude.ts");
var P = require("parsimmon");
var view_ngexpression_parser_1 = require("./view-ngexpression-parser");
;
var boolAttrHandler = {
    forAttributes: ["ng-required", "ng-disabled", "ng-trim", "ng-checked"],
    handleAttribute: function (attrName, val, allAttribs, codegenHelpers) {
        return ({ source: codegenHelpers.declareVariable("boolean", val) });
    }
};
// ng-show and ng-if introduce a scope. The reason is flow-control in typescript:
// if (variable.kind === ...) { /* typescript now knows the kind is X */ }
var boolWithScopeAttrHandler = {
    forAttributes: ["ng-show", "ng-if"],
    handleAttribute: function (attrName, val, allAttribs, codegenHelpers) {
        return ({
            source: codegenHelpers.declareVariable("boolean", val) +
                ("if (" + codegenHelpers.addScopeAccessors(val) + ") {"),
            closeSource: function () { return "}"; }
        });
    }
};
var anyAttrHandler = {
    forAttributes: ["ng-model", "ng-change", "ng-value",
        "ng-submit", "ng-class", "ng-style", "ng-init", "ng-grid"],
    handleAttribute: function (attrName, val, allAttribs, codegenHelpers) {
        return ({ source: codegenHelpers.declareVariable("any", val) });
    }
};
// attributes for which we do nothing at all.
var passThroughAttrHandler = {
    forAttributes: ["ng-form"],
    handleAttribute: function (attrName, val, allAttribs, codegenHelpers) { return ({ source: "" }); }
};
var stringAttrHandler = {
    forAttributes: ["ng-include"],
    handleAttribute: function (attrName, val, allAttribs, codegenHelpers) {
        return ({ source: codegenHelpers.declareVariable("string", val) });
    }
};
var numberAttrHandler = {
    forAttributes: ["ng-maxlength"],
    handleAttribute: function (attrName, val, allAttribs, codegenHelpers) {
        return ({ source: codegenHelpers.declareVariable("number", val) });
    }
};
var ngBindAttrDirectiveHandler = {
    forAttributes: ["ng-bind", "ng-bind-html"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        return { source: view_ngexpression_parser_1.filterExpressionToTypescript(attrValue, codegenHelpers) };
    }
};
function parseNgRepeat() {
    return view_ngexpression_parser_1.parseAtom()
        .chain(function (variable) { return view_ngexpression_parser_1.keyword("in")
        .then(view_ngexpression_parser_1.parseNgFilterExpression())
        .chain(function (expression) { return parseNgOptionsTrackBy().atMost(1)
        .map(function (trackBy) {
        var r = {
            variable: variable, expression: expression,
            trackingExpression: trackBy ? trackBy[0] : undefined
        };
        return r;
    }); }); });
}
function handleNgRepeat(attrValue, codegenHelpers) {
    var ngRepeatData = parseNgRepeat().parse(attrValue);
    if (!ngRepeatData.status) {
        console.warn("failed parsing a ng-repeat clause!");
        console.warn(attrValue);
        console.warn(ngRepeatData);
        return null;
    }
    var enumerable = view_ngexpression_parser_1.ngFilterExpressionToTypeScriptEmbedded(ngRepeatData.value.expression, codegenHelpers);
    var source = "angular.forEach(" + enumerable + ", " + codegenHelpers.registerVariable(ngRepeatData.value.variable) + " => {" +
        ("let " + codegenHelpers.registerVariable('$index') + " = 0;") +
        ("let " + codegenHelpers.registerVariable('$first') + " = true;") +
        ("let " + codegenHelpers.registerVariable('$middle') + " = true;") +
        ("let " + codegenHelpers.registerVariable('$last') + " = true;") +
        ("let " + codegenHelpers.registerVariable('$even') + " = true;") +
        ("let " + codegenHelpers.registerVariable('$odd') + " = false;") +
        (ngRepeatData.value.trackingExpression ?
            "" + codegenHelpers.declareVariable('any', ngRepeatData.value.trackingExpression) : "");
    return source;
}
var ngRepeatAttrDirectiveHandler = {
    forAttributes: ["ng-repeat"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        var source = handleNgRepeat(attrValue, codegenHelpers);
        if (source === null) {
            return { source: "" };
        }
        return {
            source: source,
            closeSource: function () { return "});"; }
        };
    }
};
var ngRepeatStartAttrDirectiveHandler = {
    forAttributes: ["ng-repeat-start"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        var source = handleNgRepeat(attrValue, codegenHelpers);
        if (source === null) {
            return { source: "" };
        }
        if (allAttribs["ng-repeat-end"] !== undefined) {
            return {
                source: source,
                closeSource: function () { return "});"; }
            };
        }
        return {
            source: source
        };
    }
};
var ngRepeatEndAttrDirectiveHandler = {
    forAttributes: ["ng-repeat-end"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        if (allAttribs["ng-repeat-start"] !== undefined) {
            return { source: "" };
        }
        return {
            source: "});"
        };
    }
};
function parseNgOptions() {
    return view_ngexpression_parser_1.parseNgFilterExpression()
        .chain(function (first) { return parseNgOptionsAs(first).or(parseNgOptionsFor({ label: first })); });
}
function maybeBracketed(next) {
    return P.regexp(/\s*\(\s*/).then(next).skip(P.regexp(/\s*\)/))
        .or(next);
}
function parseNgOptionsAs(select) {
    return view_ngexpression_parser_1.keyword("as")
        .then(maybeBracketed(view_ngexpression_parser_1.parseNgFilterExpression()))
        .chain(function (label) { return parseNgOptionsFor({ select: select, label: label }); });
}
function parseNgOptionsFor(expressions) {
    return view_ngexpression_parser_1.keyword("for")
        .then(P.takeWhile(function (c) { return c !== ' '; }).skip(view_ngexpression_parser_1.keyword("in")))
        .chain(function (value) { return view_ngexpression_parser_1.parseNgFilterExpression()
        .chain(function (array) { return parseNgOptionsTrackBy().atMost(1)
        .map(function (trackBy) {
        var r = {
            select: expressions.select,
            label: expressions.label,
            value: value,
            array: array,
            trackexpr: trackBy ? trackBy[0] : undefined
        };
        return r;
    }); }); });
}
function parseNgOptionsTrackBy() {
    return P.regexp(/\s+track\s+by\s+/).then(P.all);
}
var ngOptions = {
    forAttributes: ["ng-options"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        var ngOptionsData = parseNgOptions().parse(attrValue);
        if (!ngOptionsData.status) {
            console.warn("failed parsing a ng-options clause!");
            console.warn(attrValue);
            console.warn(ngOptionsData);
            return { source: "" };
        }
        var addVar = function (v) { return (v ? "" + codegenHelpers.declareVariable('any', v) : ""); };
        var addNgVar = function (v) {
            return (v ? view_ngexpression_parser_1.ngFilterExpressionToTypeScriptStandalone(v, codegenHelpers) : "");
        };
        var enumerable = view_ngexpression_parser_1.ngFilterExpressionToTypeScriptEmbedded(ngOptionsData.value.array, codegenHelpers);
        var source = "angular.forEach(" + enumerable + ", " + codegenHelpers.registerVariable(ngOptionsData.value.value) + " => {" +
            addNgVar(ngOptionsData.value.select) +
            addNgVar(ngOptionsData.value.label) +
            addVar(ngOptionsData.value.trackexpr) +
            "});";
        return { source: source };
    }
};
var ngWithEvent = {
    forAttributes: ["ng-blur", "ng-mouseenter", "ng-mouseleave", "ng-click"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        return { source: "const " + codegenHelpers.getNewVariableName() + ": any = (" + codegenHelpers.registerVariable('$event') + ": any) => " +
                codegenHelpers.addScopeAccessors(attrValue) + ";" };
    }
};
var ngModelOptions = {
    forAttributes: ["ng-model-options"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        var typeDef = "{updateOn?: string, debounce?: number," +
            "allowInvalid?: boolean, getterSetter?: boolean, timezone?: string}";
        return { source: codegenHelpers.declareVariable(typeDef, attrValue) };
    }
};
var ngPattern = {
    forAttributes: ["ng-pattern"],
    handleAttribute: function (attrName, attrValue, allAttribs, codegenHelpers) {
        return { source: codegenHelpers.declareVariable("RegExp|string", attrValue) };
    }
};
// ng-switch should work on the attribute level, but the spec requires to read
// multiple attributes at once... Eg "on"
var ngSwitch = {
    canHandleAttributes: ['ng-switch'],
    forTags: [],
    handleTag: function (tag, attribs, codegenHelpers) {
        if (!attribs.hasOwnProperty('ng-switch')) {
            return;
        }
        var expr = attribs.hasOwnProperty('on') ?
            attribs['on'] : attribs['ng-switch'];
        return {
            source: "switch (" + codegenHelpers.addScopeAccessors(expr) + ") {",
            closeSource: function () { return "}"; }
        };
    }
};
// ng-switch-when should work on the attribute level, but the spec requires to read
// multiple attributes at once... Eg "ng-switch-when-separator"
var ngSwitchWhen = {
    canHandleAttributes: ['ng-switch-when', 'ng-switch-when-separator', 'ng-switch-default'],
    forTags: [],
    handleTag: function (tag, attribs, codegenHelpers) {
        if (!attribs.hasOwnProperty("ng-switch-when")) {
            return;
        }
        if (attribs.hasOwnProperty("ng-switch-when-separator")) {
            var values = attribs['ng-switch-when'].split(attribs['ng-switch-when-separator']);
            var source = values.map(codegenHelpers.addScopeAccessors).map(function (v) { return "case " + v + ": break;"; }).join("");
            return { source: source };
        }
        else {
            return { source: "case " + codegenHelpers.addScopeAccessors(attribs['ng-switch-when']) + ": break;" };
        }
    }
};
var ngUiSelectDirectiveTagHandler = {
    canHandleAttributes: ['ng-model', 'allow-clear', 'ui-lock-choice'],
    forTags: ["ui-select"],
    handleTag: function (tag, attribs, codegenHelpers) {
        // a while just to introduce a new scope.
        var source = "while (1) {";
        for (var attrName in attribs) {
            var attrValue = attribs[attrName];
            switch (attrName) {
                case "ng-model":
                    source += "const " + codegenHelpers.registerVariable("$select") + " = {search:'', selected: " + codegenHelpers.addScopeAccessors(attrValue + '[0]') + "};";
                    // alright, here comes the crazy part. it seems that $item is NOT
                    // defined by ui-select.ng-model as I first thought...
                    // but actually by the subtag ui-select.ui-select-choices.repeat.
                    // which is completely nuts imho.
                    // a sub-tag is creating stuff in the scope of the parent node...
                    // anyway. creating it here in the parent scope, as null by default,
                    // overwriting it from ui-select-choices. Means it'll be T|null instead
                    // of T, but there's only so much I can do here.
                    source += "let " + codegenHelpers.registerVariable("$item") + " = null;";
                    break;
                case "allow-clear":
                    source += codegenHelpers.declareVariable("boolean", attrValue);
                    break;
                case "ui-lock-choice":
                    source += codegenHelpers.declareVariable("any", attrValue);
                    break;
            }
        }
        return { source: source, closeSource: function () { return "}"; } };
    }
};
function parseNgUiSelectChoicesSelect() {
    return view_ngexpression_parser_1.parseAtom()
        .chain(function (first) { return parseNgUiSelectChoicesAs(first)
        .or(parseNgUiSelectChoicesIn(first, prelude_ts_1.Option.none())); });
}
function parseNgUiSelectChoicesAs(varExpr) {
    return view_ngexpression_parser_1.keyword("as")
        .then(P.regexp(/[a-zA-Z0-9]+/)) // identifier
        .chain(function (identifier) { return parseNgUiSelectChoicesIn(identifier, prelude_ts_1.Option.of(varExpr)); });
}
function parseNgUiSelectChoicesIn(variable, variableExpr) {
    return view_ngexpression_parser_1.keyword("in")
        .then(view_ngexpression_parser_1.parseNgFilterExpression())
        .map(function (expression) { return ({ variable: variable, variableExpr: variableExpr, expression: expression }); });
}
var ngUiSelectChoicesTagHandler = {
    canHandleAttributes: ['repeat'],
    forTags: ["ui-select-choices"],
    handleTag: function (tag, attribs, codegenHelpers) {
        for (var attrName in attribs) {
            if (attrName === "repeat") {
                var attrValue = attribs[attrName];
                var selectData = parseNgUiSelectChoicesSelect().parse(attrValue);
                if (!selectData.status) {
                    console.warn("failed parsing a ui-select-choices select clause!");
                    console.warn(attrValue);
                    console.warn(selectData);
                    return { source: "" };
                }
                var enumerable = view_ngexpression_parser_1.ngFilterExpressionToTypeScriptEmbedded(selectData.value.expression, codegenHelpers);
                var declVar = codegenHelpers.registerVariable(selectData.value.variable);
                var variableExprSrc = selectData.value.variableExpr
                    .map(function (v) { return codegenHelpers.declareVariable('any', v); })
                    .getOrElse("");
                return {
                    // setting $item. See the comment in the ui-select handling.
                    // That also means that you should first have the ui-select-choices
                    // subtag to ui-select and then the ui-select-match otherwise it won't work.
                    source: "$item = " + enumerable + "[0];" + enumerable + ".forEach(" + declVar + " => {" + variableExprSrc,
                    closeSource: function () { return "});"; }
                };
            }
        }
        return undefined;
    }
};
/**
 * Set of angular attribute directives supported out of the box. You can give this
 * list in [[ProjectSettings.attributeDirectives]], or you can add your own or provide
 * your own list entirely.
 */
exports.defaultAttrDirectiveHandlers = [boolAttrHandler, boolWithScopeAttrHandler,
    anyAttrHandler, stringAttrHandler, numberAttrHandler,
    ngBindAttrDirectiveHandler,
    ngRepeatAttrDirectiveHandler, ngRepeatStartAttrDirectiveHandler,
    ngRepeatEndAttrDirectiveHandler,
    ngOptions, ngWithEvent,
    ngModelOptions, ngPattern, passThroughAttrHandler];
/**
 * Set of angular tag directives supported out of the box. You can give this
 * list in [[ProjectSettings.tagDirectives]], or you can add your own or provide
 * your own list entirely.
 */
exports.defaultTagDirectiveHandlers = [ngUiSelectDirectiveTagHandler, ngUiSelectChoicesTagHandler,
    ngSwitch, ngSwitchWhen];
//# sourceMappingURL=ng-directives.js.map