"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var fs_1 = require("fs");
var glob_1 = require("glob");
var prelude_ts_1 = require("prelude.ts");
var path_1 = require("path");
var view_parser_1 = require("./view-parser");
var ng_directives_1 = require("./ng-directives");
var ng_directives_2 = require("./ng-directives");
exports.defaultTagDirectiveHandlers = ng_directives_2.defaultTagDirectiveHandlers;
exports.defaultAttrDirectiveHandlers = ng_directives_2.defaultAttrDirectiveHandlers;
var controller_parser_1 = require("./controller-parser");
var filters_1 = require("./filters");
// we only repeat the imports, type synonyms and custom interfaces
// if there is a module, because otherwise those are dumped in the
// global namespace anyway
function wrapInModule(moduleName, scopeInfo, contents) {
    return "module " + moduleName + " {\n" +
        scopeInfo.imports.join("\n") + "\n" +
        scopeInfo.typeAliases.join("\n") + "\n" +
        scopeInfo.nonExportedDeclarations.join("\n") + "\n" +
        contents +
        "}\n";
}
function getViewTestFilename(ctrlFname, viewFname) {
    return ctrlFname + "_" + viewFname + "_viewtest.ts";
}
function processControllerView(prjSettings, controllerPath, viewPath, ngFilters, tagDirectives, attributeDirectives) {
    return __awaiter(this, void 0, void 0, function () {
        var scopeContents, viewExprs, pathInfo, viewPathInfo, outputFname, moduleWrap, filterParams, typeParams;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, controller_parser_1.extractControllerScopeInfo(controllerPath, prjSettings.ctrlViewFragmentExtractors)];
                case 1:
                    scopeContents = _a.sent();
                    if (scopeContents.scopeInfo.isNone()) {
                        // no point of writing anything if there is no scope block
                        return [2 /*return*/];
                    }
                    return [4 /*yield*/, view_parser_1.parseView(prjSettings.resolveImportsAsNonScope || false, viewPath, scopeContents.viewFragments, scopeContents.importNames, prelude_ts_1.Vector.ofIterable(tagDirectives), prelude_ts_1.Vector.ofIterable(attributeDirectives), prelude_ts_1.Vector.ofIterable(ngFilters))];
                case 2:
                    viewExprs = _a.sent();
                    pathInfo = path_1.parse(controllerPath);
                    viewPathInfo = path_1.parse(viewPath);
                    outputFname = pathInfo.dir + "/" +
                        getViewTestFilename(pathInfo.name, viewPathInfo.name);
                    moduleWrap = function (x) { return scopeContents.tsModuleName
                        .map(function (n) { return wrapInModule(n, scopeContents, x); })
                        .getOrElse(x); };
                    filterParams = ngFilters.map(function (f) { return "f__" + f.name + ":" + f.type; }).join(",\n    ");
                    typeParams = scopeContents.scopeTypeParams.getOrElse("");
                    fs_1.writeFileSync(outputFname, moduleWrap(scopeContents.scopeInfo.getOrThrow() +
                        ("\n\nfunction ___f" + typeParams + "($scope: Scope" + typeParams + ", " + filterParams + ") {\n") +
                        viewExprs +
                        "\n}\n") + "\n");
                    return [2 /*return*/];
            }
        });
    });
}
function deletePreviouslyGeneratedFiles(prjSettings) {
    var files = glob_1.sync(prjSettings.path + "/**/" + getViewTestFilename("*", "*"), { nodir: true, ignore: prjSettings.blacklistedPaths });
    files.forEach(function (f) { return fs_1.unlinkSync(f); });
}
/**
 * Will go through the views and controllers in the project folder and
 * generate viewtest typescript files to ascertain type-safety of the views.
 * NOTE: The function returns a promise but is not fully async: a good part of its
 * runtime is spend running synchronous functions.
 */
function processProject(prjSettings) {
    return __awaiter(this, void 0, void 0, function () {
        var files, viewInfos, viewFilenameToControllerNames, controllerNameToFilename, viewFilenameToCtrlFilenamesViewConns, viewFilenameToCtrlFilenamesModelConns, viewFilenameToCtrlFilenames;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    deletePreviouslyGeneratedFiles(prjSettings);
                    files = glob_1.sync(prjSettings.path + "/**/*.@(js|ts)", { nodir: true, ignore: prjSettings.blacklistedPaths });
                    return [4 /*yield*/, Promise.all(files.map(function (f) { return controller_parser_1.extractCtrlViewConnsAngularModule(f, prjSettings.path, prjSettings.ctrlViewConnectors, prjSettings.modelViewConnectors); }))];
                case 1:
                    viewInfos = _a.sent();
                    viewFilenameToControllerNames = prelude_ts_1.Vector.ofIterable(viewInfos)
                        .flatMap(function (vi) { return prelude_ts_1.Vector.ofIterable(vi.controllerViewInfos); })
                        .appendAll(prjSettings.extraCtrlViewConnections)
                        .groupBy(function (cvi) { return cvi.viewPath; });
                    controllerNameToFilename = prelude_ts_1.Vector.ofIterable(viewInfos)
                        .filter(function (vi) { return vi.controllerName.isSome(); })
                        .filter(function (vi) { return vi.fileName.toLowerCase().endsWith(".ts"); })
                        .toMap(function (vi) { return [vi.controllerName.getOrThrow(), vi.fileName]; });
                    viewFilenameToCtrlFilenamesViewConns = viewFilenameToControllerNames
                        .map(function (viewFname, ctrlViewInfos) {
                        return [viewFname, view_parser_1.collectionKeepDefined(ctrlViewInfos.map(function (cvi) { return controllerNameToFilename.get(cvi.controllerName).getOrUndefined(); }))];
                    });
                    viewFilenameToCtrlFilenamesModelConns = prelude_ts_1.Vector.ofIterable(viewInfos)
                        .flatMap(function (vi) { return prelude_ts_1.Vector.ofIterable(vi.modelViewInfos); })
                        .groupBy(function (mvi) { return mvi.viewPath; })
                        .mapValues(function (mvis) { return mvis.map(function (mvi) { return mvi.modelPath; }); });
                    viewFilenameToCtrlFilenames = viewFilenameToCtrlFilenamesViewConns.mergeWith(viewFilenameToCtrlFilenamesModelConns, function (views1, views2) { return views1.appendAll(views2); });
                    return [2 /*return*/, Promise.all(viewFilenameToCtrlFilenames.toVector().map(function (_a) {
                            var viewName = _a[0], ctrlNames = _a[1];
                            return Promise.all(ctrlNames.map(function (ctrlName) { return processControllerView(prjSettings, ctrlName, prjSettings.path + "/" + viewName, prjSettings.ngFilters, prjSettings.tagDirectives, prjSettings.attributeDirectives); }).toArray());
                        }).toArray())];
            }
        });
    });
}
exports.processProject = processProject;
try {
    processProject({
        path: process.argv[2],
        blacklistedPaths: process.argv.slice(3),
        ngFilters: filters_1.defaultNgFilters,
        ctrlViewConnectors: controller_parser_1.defaultCtrlViewConnectors,
        modelViewConnectors: controller_parser_1.defaultModelViewConnectors,
        extraCtrlViewConnections: [],
        tagDirectives: ng_directives_1.defaultTagDirectiveHandlers,
        attributeDirectives: ng_directives_1.defaultAttrDirectiveHandlers,
        ctrlViewFragmentExtractors: controller_parser_1.defaultCtrlViewFragmentExtractors
    });
}
catch (e) {
    console.log(e);
}
//# sourceMappingURL=ng-typeview.js.map