"use strict";
exports.__esModule = true;
var htmlparser2_1 = require("htmlparser2");
var fs_1 = require("fs");
var prelude_ts_1 = require("prelude.ts");
var view_ngexpression_parser_1 = require("./view-ngexpression-parser");
var v = 0;
function extractInlineExpressions(ngFilters, text, codegenHelpers) {
    var re = /{{(.+?)}}/g; // anything inside {{}}, multiple times
    var m;
    var result = "";
    while (m = re.exec(text)) {
        var expr = m[1];
        result += view_ngexpression_parser_1.filterExpressionToTypescript(expr, codegenHelpers);
    }
    return result;
}
/**
 * @hidden
 */
function requireDefined(x) {
    if (typeof x === "undefined") {
        throw "requireDefined(): got undefined!";
    }
    return x;
}
exports.requireDefined = requireDefined;
function collectionKeepDefined(l) {
    return l.filter(function (x) { return x !== undefined; }).map(requireDefined);
}
exports.collectionKeepDefined = collectionKeepDefined;
function listKeepDefined(l) {
    return l.filter(function (x) { return x !== undefined; }).map(requireDefined);
}
exports.listKeepDefined = listKeepDefined;
/**
 * http://stackoverflow.com/a/16184477/516188
 * @hidden
 */
function normalizeTagAttrName(name) {
    return name
        .replace("_", "-")
        .replace(":", "-")
        .replace(/^x\-/, "")
        .replace(/^data\-/, "")
        .replace(/([A-Z])/g, function (l) { return "-" + l.toLowerCase(); });
}
exports.normalizeTagAttrName = normalizeTagAttrName;
function handleDirectiveResponses(xpath, codegenHelpers, resps) {
    return resps
        .filter(function (x) { return x.closeSource !== undefined ||
        codegenHelpers.ngScopeInfo.curScopeVars.length > 0; })
        .map(function (r) { return ({
        xpathDepth: xpath.length(),
        closeSource: r.closeSource || (function () { return ""; }),
        variables: codegenHelpers.ngScopeInfo.curScopeVars
    }); });
}
function getHandler(fileName, defaultScope, tagDirectiveHandlers, attrDirectiveHandlers, ngFilters, f) {
    var expressions = "";
    var xpath = prelude_ts_1.Vector.of();
    var activeScopes = prelude_ts_1.Vector.ofIterable([{
            xpathDepth: 0,
            closeSource: function () { return ""; },
            variables: defaultScope
        }]);
    var getNewVariableName = function () { return "___x" + v++; };
    return {
        onopentag: function (_name, _attribs) {
            var name = normalizeTagAttrName(_name);
            var attribs = {};
            for (var k in _attribs) {
                attribs[normalizeTagAttrName(k)] = _attribs[k];
            }
            xpath = xpath.append(name);
            // work on tag handlers
            var codegenHelpersTag = new view_ngexpression_parser_1.CodegenHelper(ngFilters, activeScopes, getNewVariableName);
            if (tagDirectiveHandlers
                .filter(function (d) { return d.forTags.indexOf(name) >= 0; }).isEmpty() && name.startsWith("ng-")) {
                console.warn("Warning: unhandled tag: " + name);
            }
            var relevantTagHandlers = tagDirectiveHandlers
                .filter(function (d) { return d.forTags.length === 0 || d.forTags.indexOf(name) >= 0; });
            var tagDirectiveResps = listKeepDefined(relevantTagHandlers.map(function (handler) { return handler.handleTag(name, attribs, codegenHelpersTag); }));
            expressions += tagDirectiveResps.map(function (x) { return x.source; }).mkString("");
            activeScopes = activeScopes.appendAll(handleDirectiveResponses(xpath, codegenHelpersTag, tagDirectiveResps));
            var _loop_1 = function (attrName) {
                var codegenHelpersAttr = new view_ngexpression_parser_1.CodegenHelper(ngFilters, activeScopes, getNewVariableName);
                var attrValue = attribs[attrName];
                var handlers = attrDirectiveHandlers
                    .filter(function (d) { return d.forAttributes.indexOf(attrName) >= 0; });
                if (!handlers.isEmpty()) {
                    var attrDirectiveResps = listKeepDefined(handlers.map(function (handler) { return handler.handleAttribute(attrName, attrValue, attribs, codegenHelpersAttr); }));
                    expressions += attrDirectiveResps.map(function (x) { return x.source; }).mkString("");
                    activeScopes = activeScopes.appendAll(handleDirectiveResponses(xpath, codegenHelpersAttr, attrDirectiveResps));
                }
                else if (attrName.startsWith("ng-") &&
                    !relevantTagHandlers.find(function (th) { return th.canHandleAttributes.indexOf(attrName) >= 0; })) {
                    console.warn("Warning: unhandled attribute: " + attrName);
                }
                expressions += extractInlineExpressions(ngFilters, attrValue, codegenHelpersAttr);
            };
            // work on attribute handlers
            for (var attrName in attribs) {
                _loop_1(attrName);
            }
        },
        onclosetag: function (name) {
            if (xpath.last().getOrUndefined() !== name) {
                console.error(fileName + ": expected </" + xpath.last().getOrUndefined() + "> but found </" + name + ">");
            }
            xpath = xpath.dropRight(1);
            var firstScope = activeScopes.last().getOrUndefined();
            while (firstScope && firstScope.xpathDepth > xpath.length()) {
                expressions += firstScope.closeSource();
                activeScopes = activeScopes.dropRight(1);
                firstScope = activeScopes.last().getOrUndefined();
            }
        },
        ontext: function (text) {
            var codegenHelpers = new view_ngexpression_parser_1.CodegenHelper(ngFilters, activeScopes, getNewVariableName);
            expressions = expressions.concat(extractInlineExpressions(ngFilters, text, codegenHelpers));
        },
        onend: function () {
            f(indentSource(expressions));
        }
    };
}
function indentSource(src) {
    var multiple = 4;
    var depth = 1;
    var inSingleQuotes = false;
    var inDoubleQuotes = false;
    var previousIsEndBlock = false;
    var addCr = function () { return "\n" + " ".repeat(depth * multiple); };
    var result = " ".repeat(depth * multiple);
    for (var i = 0; i < src.length; i++) {
        var chr = src[i];
        if (previousIsEndBlock && [';', ')'].indexOf(chr) < 0) {
            result += addCr();
        }
        previousIsEndBlock = false;
        if (inSingleQuotes) {
            if (chr === "'") {
                inSingleQuotes = false;
            }
            result += chr;
        }
        else if (inDoubleQuotes) {
            if (chr === '"') {
                inDoubleQuotes = false;
            }
            result += chr;
        }
        else {
            if (chr === "'") {
                result += "'";
                inSingleQuotes = true;
            }
            else if (chr === '"') {
                result += '"';
                inDoubleQuotes = true;
            }
            else if (chr === ';') {
                result += ";" + addCr();
            }
            else if (chr === '{') {
                ++depth;
                result += "{" + addCr();
            }
            else if (chr === '}') {
                --depth;
                result += addCr() + "}";
                previousIsEndBlock = true;
            }
            else {
                result += chr;
            }
        }
    }
    return result;
}
/**
 * @hidden
 */
function parseView(resolveImportsAsNonScope, fileName, viewFragments, importNames, tagDirectiveHandlers, attrDirectiveHandlers, ngFilters) {
    var defaultScope = resolveImportsAsNonScope ? importNames : [];
    return new Promise(function (resolve, reject) {
        var parser = new htmlparser2_1.Parser(getHandler(fileName, defaultScope, tagDirectiveHandlers, attrDirectiveHandlers, ngFilters, resolve));
        parser.write(fs_1.readFileSync(fileName).toString());
        viewFragments.forEach(function (f) { return parser.write(f); });
        parser.done();
    });
}
exports.parseView = parseView;
//# sourceMappingURL=view-parser.js.map