"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
exports.__esModule = true;
var assert = require("assert");
var prelude_ts_1 = require("prelude.ts");
var controller_parser_1 = require("../src/controller-parser");
var ts = require("typescript");
var ctrlViewConn = {
    interceptAstNode: ts.SyntaxKind.CallExpression,
    getControllerView: function (node, projectPath) {
        var call = node;
        if (["displayDialog", "core.displayDialog"].indexOf(call.expression.getText()) < 0) {
            return [];
        }
        if (call.arguments.length < 3) {
            return [];
        }
        if (call.arguments[1].kind !== ts.SyntaxKind.StringLiteral ||
            call.arguments[2].kind !== ts.SyntaxKind.StringLiteral) {
            return [];
        }
        return [{
                controllerName: call.arguments[1].text,
                viewPath: call.arguments[2].text
            }];
    }
};
describe("extractModalOpenAngularModule", function () {
    it("should recognize the statements", function () { return __awaiter(_this, void 0, void 0, function () {
        var modalModuleInfo;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, controller_parser_1.extractCtrlViewConnsAngularModule("test/data/test-ctrl.ts", "webapp", controller_parser_1.defaultCtrlViewConnectors.concat([ctrlViewConn]), controller_parser_1.defaultModelViewConnectors)];
                case 1:
                    modalModuleInfo = _a.sent();
                    assert.equal("test/data/test-ctrl.ts", modalModuleInfo.fileName);
                    assert.deepEqual(prelude_ts_1.Option.of("my.ng.module.name"), modalModuleInfo.ngModuleName);
                    assert.deepEqual(prelude_ts_1.Option.of("ControllerName"), modalModuleInfo.controllerName);
                    assert.deepEqual([
                        {
                            controllerName: "CtrlState1",
                            viewPath: "app/view/url1.html"
                        },
                        {
                            controllerName: "CtrlState2",
                            viewPath: "app/view/url2.html"
                        },
                        {
                            controllerName: "ControllerName",
                            viewPath: "test-view.html"
                        },
                        {
                            controllerName: "AnotherControllerName",
                            viewPath: "path/to/another/view.html"
                        },
                        {
                            controllerName: "YupYetAnotherCtrl",
                            viewPath: "and/yet/another/view.html"
                        }
                    ], modalModuleInfo.controllerViewInfos);
                    return [2 /*return*/];
            }
        });
    }); });
});
describe("extractControllerScopeInfo", function () {
    it("should parse the scope info", function () { return __awaiter(_this, void 0, void 0, function () {
        var scopeInfo;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, controller_parser_1.extractControllerScopeInfo("test/data/test-ctrl.ts", [])];
                case 1:
                    scopeInfo = _a.sent();
                    assert.equal("multipart.module.name", scopeInfo.tsModuleName.getOrThrow());
                    assert.equal("interface Scope extends ng.IScope {\n" +
                        "        showDiv?: string;\n" +
                        "        showText: (x:string)=>boolean;\n" +
                        "        data: {groups: any[], firstname: string}\n" +
                        "        triggerAction: boolean\n" +
                        "        user: string;\n" +
                        "        maxlength: number;\n" +
                        "        boolean1: boolean;\n" +
                        "        boolean2: boolean;\n" +
                        "        boolean3: boolean;\n" +
                        "    }", scopeInfo.scopeInfo.getOrThrow());
                    assert.deepEqual(["type STR = string;", "type INT = number;"], scopeInfo.typeAliases);
                    assert.deepEqual(["import Aa = api.Aa;", "import Bb = api.Bb;"], scopeInfo.imports);
                    assert.deepEqual([
                        "interface NotScope extends SomethingElse {\n        intField: number;\n    }",
                        "interface NotScope2 extends NotScope {\n        f1: (x:string)=>boolean;\n    }",
                        "class NotScopeClass {\n        field?: number;\n        constructor(public f2: number);\n    }"
                    ], scopeInfo.nonExportedDeclarations);
                    return [2 /*return*/];
            }
        });
    }); });
});
//# sourceMappingURL=controller-parser.js.map